#include <gtk/gtk.h>
#include <stdio.h>
#include <stdarg.h>
#include "blursk.h"

typedef struct
{
	gboolean  open;
	GtkWidget *window;
	GtkWidget *close;
	GtkWidget *text;
} dialog_t;
static dialog_t about_dialog;
static dialog_t error_dialog;

static void close_cb(GtkWidget *w, gpointer data)
{
	if (about_dialog.open &&
		(w == about_dialog.window || w == about_dialog.close))
	{
		gtk_widget_destroy(about_dialog.window);
		about_dialog.open = FALSE;
	}
	else if (error_dialog.open &&
		(w == error_dialog.window || w == error_dialog.close))
	{
		gtk_widget_destroy(error_dialog.window);
		error_dialog.open = FALSE;
	}
}

static void addtext(GtkTextBuffer *twidget, const char *text)
{
	GtkTextIter iter;
	gchar titlechar[2];
	const char *body;

	memset(titlechar, 0, sizeof titlechar);
	gtk_text_buffer_create_tag (twidget, "TITLEFONT",
				     "family", "sans",
				     "size", 16 * PANGO_SCALE,
				     "weight", PANGO_WEIGHT_BOLD, NULL);  
				        
	gtk_text_buffer_create_tag (twidget, "TEXTFONT",
				     "family", "courier",
				     "size", 12 * PANGO_SCALE, NULL);

	gtk_text_buffer_get_iter_at_offset (twidget, &iter, 0);
	/* Add the first line in the "TITLEFONT" */
	for (body = text; *body && *body != '\n'; body++)
	{
		titlechar[0] = *body;
		gtk_text_buffer_insert_with_tags_by_name (twidget, &iter,
			titlechar, -1,
			"TITLEFONT", NULL);
	}
	
	if (*body == '\n')
	{
		*body++;
	}
	
	/* If there's any remaining text, add it in "TEXTFONT" */
	if (*body)
	{
		gtk_text_buffer_insert_with_tags_by_name (twidget, &iter,
			 body, -1,
			 "TEXTFONT", NULL);
	}
}

static dialog_t showtext(const char *text, const char *close_label)
{
	GtkWidget *vbox, *scroll, *bbox, *btn;
	dialog_t ret;
    
	/* Create a new window */
	ret.window = gtk_dialog_new ();
	vbox = GTK_DIALOG (ret.window)->vbox;
	gtk_window_set_position(GTK_WINDOW(ret.window), GTK_WIN_POS_MOUSE);
	gtk_window_set_default_size (GTK_WINDOW (ret.window), 640, 400);
	
	/* Create a scrolled window in the vbox */
	scroll = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);

	/* Create a text widget in the scrolled window */
	ret.text = gtk_text_view_new ();
	gtk_container_add (GTK_CONTAINER (scroll), ret.text);
	gtk_text_view_set_editable (GTK_TEXT_VIEW (ret.text), FALSE);
	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (ret.text), FALSE);
	addtext(gtk_text_view_get_buffer (GTK_TEXT_VIEW (ret.text)), text);
		
	/* Add a [Close] button */
	ret.close = gtk_button_new_with_label(close_label);
	g_signal_connect_object((gpointer)(ret.close), "clicked", 
		GTK_SIGNAL_FUNC(close_cb), GTK_OBJECT(ret.window), 0);
	GTK_WIDGET_SET_FLAGS(ret.close, GTK_CAN_DEFAULT);
	gtk_dialog_add_action_widget (GTK_DIALOG (ret.window), ret.close, GTK_RESPONSE_OK);
	gtk_widget_grab_default(ret.close);

	/* Also make the window manager's "close" button close the window */
	g_signal_connect_object((gpointer)(ret.window), "delete_event",
		 GTK_SIGNAL_FUNC(close_cb), GTK_OBJECT(ret.window), 0);

	/* Show everything */
	gtk_widget_show(ret.close);
	gtk_widget_show(ret.text);
	gtk_widget_show(scroll);
	gtk_widget_show(vbox);
	gtk_widget_show(ret.window);

	ret.open = TRUE;
	return ret;
}

void about()
{
	/* If already showing the window, do nothing */
	if (about_dialog.open)
		return;

	about_dialog = showtext(readme, "Close");
}

void about_error(char *format, ...)
{
	char	buf[2000];
	va_list	args;

	/* format the error message */
	va_start(args, format);
	vsprintf(buf, format, args);
	va_end(args);

	/* if no error window yet, then create one */
	if (error_dialog.open)
		addtext(gtk_text_view_get_buffer (GTK_TEXT_VIEW (error_dialog.text)), buf);
	else
		error_dialog = showtext(buf, "Shit Happens");
}

/*main(int argc, char **argv)
{
	gtk_init(&argc, &argv);
	about();
	gtk_main();
	return 0;
}*/
