#include "config.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
//#include <gtk-
#include <audacious/plugin.h>
#include <audacious/beepctrl.h>
#include <audacious/util.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "sockets.h"

#define TIME_UNIT 125000

int sock;
int broken = 0;
int playlist_pos;
long time_v;
int showing_vol=0, last_vol = 0;

static void lcd_init(void);
static void lcd_cleanup(void);
static void lcd_playback_start(void);
static void lcd_playback_stop(void);
static void lcd_about(void);
static void lcd_update(gint16 data[2][256]);
char *get_token(char *buf);

VisPlugin lcd_vp =
{
	NULL,
	NULL,
	0,
	NULL, /* Description */
	0,
	1,		
	lcd_init, /* init */
	lcd_cleanup, /* cleanup */
	lcd_about, /* about */
	NULL, /* configure */
	NULL, /* disable_plugin */
	lcd_playback_start, /* playback_start */
	lcd_playback_stop, /* playback_stop */
	NULL, /* render_pcm */
	lcd_update  /* render_freq */
};

VisPlugin *get_vplugin_info(void)
{
	lcd_vp.description =
                g_strdup_printf("MiniLCD plugin v.%s", VERSION);
	return &lcd_vp;
}

// shows a small intro-screen
static void lcd_aboutp(gint showing)
{

if(broken) return;

if (showing) 
    {
	sock_send_string(sock, "widget_add xmms name string\n");
	sock_send_string(sock, "widget_set xmms name 2 1 {About miniLCD:}\n");

	sock_send_string(sock, "widget_add xmms about scroller\n");
	sock_send_string(sock, "widget_set xmms about 1 2 16 2 h 3 {miniLCD visualization plugin. Brought to you by Wit Wilinski <madman@linux.bydg.org>}\n");
    } else {
	sock_send_string(sock, "widget_del xmms name\n");
	sock_send_string(sock, "widget_del xmms about\n");
    }    	
}




static void lcd_about(void)
{
/*	GtkWidget *dialog;
	dialog = xmms_show_message("About MiniLCD", "MiniLCD plugin v."
					VERSION 
					"\nA plugin based on Kuro's LCDxmms\n"
					"Displays some song info on a 16x2 LCD.\n\n"
					"Written by: Wit Wiliski <madman@linux.bydg.org>\n"
					"[LCDxmms by kuro <kuro@fused.org>]", 
					"all right, man", FALSE, NULL, NULL);
	gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
				GTK_SIGNAL_FUNC(gtk_widget_destroyed),
				&dialog);
*/
}


static void lcd_init(void)
{
	int len;
	int count = 0;
	char buf[8192];
	char *token = NULL;
	char arg[20];
	char cmd[80];

	sock = sock_connect("localhost", LCDPORT);
	if(sock<=0)
	{
		printf("Error connecting to localhost on port LCDPORT.\n");
		broken = 1;
		return;
	}
	sock_send_string(sock, "hello\n");

	do
	{
		usleep(TIME_UNIT);
		len = sock_recv(sock, buf, 8000);
		count++;
	} while (len <= 0 && count < 50);

	if(len <= 0)
	{
		printf("Couldn't communicate with server!\n");
		broken = 1;
		return;
	}

	sock_send_string(sock, "client_set name {MiniLCD}\n");
	sock_send_string(sock, "screen_add xmms\n");
	sock_send_string(sock, "screen_set xmms name {LCD}\n");
	sock_send_string(sock, "screen_set xmms heartbeat off\n");
	    lcd_aboutp(1);
}

static void lcd_cleanup(void)
{
	sock_close(sock);
}


static void lcd_playback_start(void)
{
	char cmd[240];
	int playlist_pos;
	char *song_title;

	if(broken) return;

	playlist_pos = xmms_remote_get_playlist_pos(0);
	song_title = xmms_remote_get_playlist_title(0, playlist_pos);
	
	lcd_aboutp(0);

	sock_send_string(sock, "widget_add xmms tclock string\n");
   sock_send_string(sock, "widget_add xmms eclock string\n"); 
	sock_send_string(sock, "widget_add xmms shuffle string\n");
	sock_send_string(sock, "widget_add xmms repeat string\n");	
	sock_send_string(sock, "widget_set xmms eclock 6 2 {00:00/}\n");
	sock_send_string(sock, "widget_set xmms tclock 12 2 {00:00}\n");
	sock_send_string(sock, "widget_set xmms shuffle 1 2 {  }\n");
	sock_send_string(sock, "widget_set xmms repeat 3 2 {  }\n");	

	sock_send_string(sock, "widget_add xmms song scroller\n");
	sock_send_string(sock, cmd); 

	sprintf(cmd, "widget_set xmms song 1 1 16 1 v 8 {%s}\n", song_title);
	sock_send_string(sock, cmd); 	
}


static void lcd_playback_stop(void)
{
	if(broken) return;
	sock_send_string(sock, "widget_del xmms song\n");
	sock_send_string(sock, "widget_del xmms eclock\n");
   sock_send_string(sock, "widget_del xmms eclock\n");
	sock_send_string(sock, "widget_del xmms shuffle\n");
	sock_send_string(sock, "widget_del xmms repeat\n");
	lcd_aboutp(1);
}


static void lcd_update(gint16 data[2][256])
{
	char buf[8192];
	char cmd[240];
	char *rep, *shf;
	char *song_title;
	int len, pos;
	int time_code, time_min, time_sec, track_time;
	int vol;


	if(broken) return;

	len = sock_recv(sock, buf, 8000);  //dont want the pipes to clog

	if(xmms_remote_get_playlist_pos(0) != playlist_pos)
	{
		playlist_pos = xmms_remote_get_playlist_pos(0);
		song_title = xmms_remote_get_playlist_title(0, playlist_pos);
		sprintf(cmd, "widget_set xmms song 1 1 16 1 v 8 {%s}\n", song_title);
		sock_send_string(sock, cmd); 
	}

        track_time=xmms_remote_get_playlist_time(0,playlist_pos)/1000;
	time_code = xmms_remote_get_output_time(0) / 1000;
	time_min = time_code / 60;
	time_sec = time_code % 60;

	if((xmms_remote_is_shuffle(0)!=TRUE)) { shf="  ";} else { shf="Sh"; }
	if((xmms_remote_is_repeat(0)!=TRUE)) { rep="  ";} else { rep="Rp"; }
	
	sprintf(cmd, "widget_set xmms repeat 3 2 {%s}\n", rep);
	sock_send_string(sock, cmd);

	sprintf(cmd, "widget_set xmms shuffle 1 2 {%s}\n", shf);
	sock_send_string(sock, cmd);
   
        time_min = time_code / 60;                                              
           time_sec = time_code % 60;
	sprintf(cmd, "widget_set xmms eclock 6 2 {%.2d:%.2d/}\n", time_min, time_sec);
	sock_send_string(sock, cmd);

           time_min = track_time / 60;                                             
                time_sec = track_time % 60;                                           
           sprintf(cmd, "widget_set xmms tclock 12 2 {%.2d:%.2d}\n", time_min, time_sec);
		           sock_send_string(sock, cmd); 
   
// my new volume-hack ;)

	vol = xmms_remote_get_main_volume(0);
    if (vol!=last_vol) {
	if (!showing_vol) {
	 time_v=time(NULL);
	 sock_send_string(sock, "widget_add xmms vol string\n");		
	 showing_vol=1;
	 last_vol=vol; 
     }
    
    if (vol==100) { sprintf(cmd, "widget_set xmms vol 1 2 {[ Volume : Max ]}\n"); }
     else { sprintf(cmd, "widget_set xmms vol 1 2 {[ Volume : %2d% ]}\n", vol); }
	sock_send_string(sock, cmd);
    
    }
    

    if (showing_vol && (time_v+2<time(NULL))) {
	showing_vol=0;
	sock_send_string(sock, "widget_del xmms vol\n");
    }
	
}
